package com.example.pibd22_kalyshev_y_v_motorboat_hard;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;

import java.util.Random;

public class ControllerBoat {
    @FXML
    private Pane root;
    @FXML
    private Canvas canvas;
    @FXML
    private FlowPane flowPane;
    @FXML
    private Label labelSpeedValue;
    @FXML
    private Label labelWeightValue;
    @FXML
    private Label labelBodyColorValue;
    @FXML
    private ComboBox<String> comboBoxNumOfRollers;
    @FXML
    private ComboBox<String> comboBoxOrnamentType;
    @FXML
    private Button buttonCreate;
    @FXML
    private Button buttonCreateModif;
    @FXML
    private Button buttonLeft;
    @FXML
    private Button buttonRight;
    @FXML
    private Button buttonUp;
    @FXML
    private Button buttonDown;
    private final double rootPadding = 10.0;
    private DrawningBoat _boat;

    @FXML
    public void initialize()
    {
        buttonCreate.setTranslateX(rootPadding);

        root.widthProperty().addListener((obs, oldVal, newVal) ->
        {
            UpdateGUI();
            if (_boat != null)
            {
                _boat.ChangeBorders((int) canvas.getWidth(), (int) canvas.getHeight());
            }
            Draw();
        });
        root.heightProperty().addListener((obs, oldVal, newVal) ->
        {
            UpdateGUI();
            if (_boat != null)
            {
                _boat.ChangeBorders((int) canvas.getWidth(), (int) canvas.getHeight());
            }
            Draw();
        });
    }

    @FXML
    void ButtonCreate_Click()
    {
        Random rnd = new Random();
        _boat = new DrawningBoat(rnd.nextInt(200) + 100, rnd.nextInt(1000) + 1000,
                Color.rgb(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)));
        SetData();
        Draw();
    }

    @FXML
    void ButtonCreateModif_Click()
    {
        Random rnd = new Random();
        _boat = new DrawningSpeedboat(rnd.nextInt(200) + 100, rnd.nextInt(1000) + 1000,
                Color.rgb(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)),
                Color.rgb(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)),
                (rnd.nextInt(2) != 0), (rnd.nextInt(2) != 0), (rnd.nextInt(2) != 0));
        SetData();
        Draw();
    }

    @FXML
    void ButtonMove_Click(ActionEvent event)
    {
        if (_boat == null)
        {
            return;
        }
        String buttonName = ((Button) event.getSource()).getId();
        switch (buttonName)
        {
            case "buttonUp" -> _boat.MoveTransport(Direction.Up);
            case "buttonDown" -> _boat.MoveTransport(Direction.Down);
            case "buttonLeft" -> _boat.MoveTransport(Direction.Left);
            case "buttonRight" -> _boat.MoveTransport(Direction.Right);
        }
        Draw();
    }

    @FXML
    private void ComboBoxOrnamentType_Changed()
    {
        if (_boat != null)
        {
            ChangeDrawningOars();
            Draw();
        }
    }

    @FXML
    private void ComboBoxNumOfRollers_Changed()
    {
        if (_boat != null)
        {
            _boat.GetDrawningOars().SetNumberOars(Integer.parseInt(comboBoxNumOfRollers.getValue()));
            Draw();
        }
    }

    private void ChangeDrawningOars()
    {
        if (_boat != null)
        {
            IDrawningAdditionalElement newDrawningTrackRollers = switch (comboBoxOrnamentType.getValue())
                    {
                        case "None" -> new DrawningOars(_boat.GetBoat().GetBodyColor());
                        case "Dots" -> new DrawningDotOrnamentTrackRollers(_boat.GetBoat().GetBodyColor());
                        case "Circles" ->
                                new DrawningCircleOrnamentTrackRollers(_boat.GetBoat().GetBodyColor());
                        default -> null;
                    };
            if (newDrawningTrackRollers != null)
            {
                _boat.SetDrawningOars(newDrawningTrackRollers);
                _boat.GetDrawningOars().SetNumberOars(Integer.parseInt(comboBoxNumOfRollers.getValue()));
            }
        }
    }

    private void UpdateGUI()
    {
        double rootWidth = root.getWidth();
        double rootHeight = root.getHeight();

        double flowPaneHeight = flowPane.getHeight();
        double buttonCreateHeight = buttonCreate.getHeight();

        canvas.setWidth(rootWidth);
        flowPane.setPrefWidth(rootWidth);
        canvas.setHeight(rootHeight - flowPaneHeight);
        flowPane.setTranslateY(rootHeight - flowPaneHeight);

        buttonCreate.setTranslateY(rootHeight - flowPaneHeight - buttonCreateHeight - rootPadding);
        buttonCreateModif.setTranslateY(rootHeight - flowPaneHeight - buttonCreateHeight - rootPadding);

        int buttonMoveSize = 30;
        int distanceBetweenButtons = 5;
        buttonUp.setTranslateY(rootHeight - flowPaneHeight - buttonMoveSize * 2.0 - rootPadding -
                distanceBetweenButtons);
        buttonUp.setTranslateX(rootWidth - rootPadding - buttonMoveSize * 2.0 - distanceBetweenButtons);

        buttonDown.setTranslateY(rootHeight - flowPaneHeight - buttonMoveSize - rootPadding);
        buttonDown.setTranslateX(rootWidth- rootPadding - buttonMoveSize * 2.0 - distanceBetweenButtons);

        buttonLeft.setTranslateY(rootHeight - flowPaneHeight - buttonMoveSize - rootPadding);
        buttonLeft.setTranslateX(rootWidth - rootPadding - buttonMoveSize * 3.0 -
                distanceBetweenButtons * 2.0);

        buttonRight.setTranslateY(rootHeight - flowPaneHeight - buttonMoveSize - rootPadding);
        buttonRight.setTranslateX(rootWidth - rootPadding - buttonMoveSize);
    }

    private void Draw()
    {
        GraphicsContext gc = canvas.getGraphicsContext2D();

        gc.clearRect(0.0, 0.0, canvas.getWidth(), canvas.getHeight());
        gc.setFill(Color.WHITE);
        gc.fillRect(0.0, 0.0, canvas.getWidth(), canvas.getHeight());

        gc.setStroke(Color.BLACK);
        gc.setLineWidth(4);
        gc.strokeRect(0.0, 0.0, canvas.getWidth(), canvas.getHeight());

        if (_boat != null)
        {
            _boat.DrawTransport(gc);
        }
    }

    private void SetData()
    {
        ChangeDrawningOars();
        Random rnd = new Random();
        _boat.SetPosition(rnd.nextInt(90) + 10, rnd.nextInt(90),
                (int) canvas.getWidth(), (int) canvas.getHeight());
        labelSpeedValue.setText(Integer.toString(_boat.GetBoat().GetSpeed()));
        labelWeightValue.setText(Double.toString(_boat.GetBoat().GetWeight()));
        labelBodyColorValue.setText(_boat.GetBoat().GetBodyColor().toString());
    }
}
